==> nmrium-0.53.0.tgz: package/lib/component/modal/editRange/forms/components/signal-tabs/SignalJCouplingsTable.js <==
import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { useFormikContext } from 'formik';
import { translateMultiplet } from 'nmr-processing';
import { useCallback, useMemo, useState } from 'react';
import { FaPlus, FaRegTrashAlt } from 'react-icons/fa';
import { Toolbar } from 'react-science/ui';
import { Multiplets } from '../../../../../../data/constants/Multiplets';
import { isSpectrum1D } from '../../../../../../data/data1d/Spectrum1D';
import { useChartData } from '../../../../../context/ChartContext';
import Button from '../../../../../elements/Button';
import ReactTable from '../../../../../elements/ReactTable/ReactTable';
import FormikInput from '../../../../../elements/formik/FormikInput';
import FormikSelect from '../../../../../elements/formik/FormikSelect';
import { usePanelPreferences } from '../../../../../hooks/usePanelPreferences';
import useSpectrum from '../../../../../hooks/useSpectrum';
import { hasCouplingConstant } from '../../../../../panels/extra/utilities/MultiplicityUtilities';
import { useEvent } from '../../../../../utility/Events';
import { formatNumber } from '../../../../../utility/formatNumber';
const styles = {
    input: {
        width: '100%',
        padding: '0.25rem 0.5rem',
    },
    select: {
        height: '26px',
        width: '100%',
    },
    column: {
        padding: '2px',
    },
};
function getJCouplingKey(signalIndex, jIndex, jCouplingKey) {
    return `signals[${signalIndex}].js.${jIndex}.${jCouplingKey}`;
}
export function SignalJCouplingsTable(props) {
    const { values, setFieldValue } = useFormikContext();
    const signal = values?.signals?.[values?.signalIndex] || {};
    const jCouplings = signal?.js || [];
    const [lastSelectedCouplingIndex, setLastSelectedCouplingIndex] = useState(null);
    const spectrum = useSpectrum();
    const { view: { spectra: { activeTab }, }, } = useChartData();
    const rangesPreferences = usePanelPreferences('ranges', activeTab);
    useEvent({
        onClick: (options) => {
            if (`${props.index}` === values.signalIndex &&
                typeof lastSelectedCouplingIndex === 'number') {
                const x = formatNumber(options.xPPM, rangesPreferences.deltaHz.format);
                void setFieldValue(getJCouplingKey(values.signalIndex, lastSelectedCouplingIndex, 'coupling'), x);
            }
        },
        onBrushEnd: (options) => {
            const { range: [from, to], } = options;
            if (`${props.index}` === values.signalIndex &&
                typeof lastSelectedCouplingIndex === 'number' &&
                isSpectrum1D(spectrum)) {
                const value = Number(formatNumber(Math.abs(to - from) * spectrum.info.originFrequency, rangesPreferences.deltaHz.format));
                void setFieldValue(getJCouplingKey(values.signalIndex, lastSelectedCouplingIndex, 'coupling'), value);
            }
        },
    });
    const addHandler = useCallback((data) => {
        const coupling = {
            multiplicity: translateMultiplet('m'),
            coupling: '',
        };
        void setFieldValue(`signals[${values?.signalIndex}].js`, [
            ...data,
            coupling,
        ]);
        setLastSelectedCouplingIndex(data.length);
    }, [setFieldValue, values?.signalIndex]);
    const deleteHandler = useCallback((data, index) => {
        const jCouplings = data.filter((_, columnIndex) => columnIndex !== index);
        void setFieldValue(`signals[${values?.signalIndex}].js`, jCouplings);
        if (lastSelectedCouplingIndex === index) {
            setLastSelectedCouplingIndex(null);
        }
    }, [lastSelectedCouplingIndex, setFieldValue, values?.signalIndex]);
    function deleteAllHandler() {
        void setFieldValue(`signals[${values?.signalIndex}].js`, []);
        setLastSelectedCouplingIndex(null);
    }
    const multiplicityChangeHandler = useCallback((value, name) => {
        if (!hasCouplingConstant(value)) {
            void setFieldValue(name, '');
        }
    }, [setFieldValue]);
    const COLUMNS = useMemo(() => [
        {
            Header: '#',
            style: { width: '25px', ...styles.column },
            accessor: (_, index) => index + 1,
        },
        {
            Header: 'Multiplicity',
            style: { padding: 0, ...styles.column },
            Cell: ({ row }) => {
                return (_jsx(FormikSelect, { className: "select-box", name: getJCouplingKey(values.signalIndex, row.index, 'multiplicity'), items: Multiplets, itemValueField: "label", onChange: (value) => {
                        multiplicityChangeHandler(value, getJCouplingKey(values.signalIndex, row.index, 'coupling'));
                    }, style: styles.select, onClick: (e) => e.stopPropagation() }));
            },
        },
        {
            Header: 'J (Hz)',
            style: { padding: 0, ...styles.column },
            Cell: ({ row }) => {
                return (_jsx(FormikInput, { name: getJCouplingKey(values.signalIndex, row.index, 'coupling'), type: "number", placeholder: 'J (Hz)', style: { input: styles.input }, disabled: !hasCouplingConstant(row.original?.multiplicity), onClick: (e) => e.stopPropagation() }));
            },
        },
        {
            Header: '',
            style: { width: '70px', ...styles.column },
            id: 'action-button',
            Cell: ({ data, row }) => {
                const record = row.original;
                return (_jsx("div", { style: { display: 'flex', justifyContent: 'space-around' }, children: !record?.name && (_jsx(Button.Danger, { fill: "outline", onClick: (e) => {
                            e.stopPropagation();
                            deleteHandler(data, row.index);
                        }, children: _jsx(FaRegTrashAlt, {}) })) }));
            },
        },
    ], [deleteHandler, multiplicityChangeHandler, values.signalIndex]);
    function selectRowHandler(index) {
        setLastSelectedCouplingIndex((prevIndex) => prevIndex === index ? null : index);
    }
    function handleActiveRow(row) {
        return row?.index === lastSelectedCouplingIndex;
    }
    return (_jsxs("div", { style: { display: 'flex', flexDirection: 'column' }, children: [_jsx("div", { style: { padding: '5px 0', display: 'flex' }, children: _jsxs(Toolbar, { children: [_jsx(Toolbar.Item, { icon: _jsx(FaPlus, {}), title: "Add a new J coupling", intent: "success", onClick: () => addHandler(jCouplings) }), _jsx(Toolbar.Item, { icon: _jsx(FaRegTrashAlt, {}), title: "Delete all J couplings", intent: "danger", onClick: deleteAllHandler })] }) }), _jsx("div", { style: {
                    maxHeight: '300px',
                    overflow: 'auto',
                }, children: _jsx(ReactTable, { data: jCouplings, columns: COLUMNS, onClick: (e, rowData) => selectRowHandler(rowData.index), activeRow: handleActiveRow, emptyDataRowText: "No J Couplings" }) })] }));
}

==> nmrium-0.54.0.tgz: package/lib/component/modal/editRange/forms/components/signal-tabs/SignalJCouplingsTable.js <==
import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { useFormikContext } from 'formik';
import { translateMultiplet } from 'nmr-processing';
import { useCallback, useMemo, useState } from 'react';
import { FaPlus, FaRegTrashAlt } from 'react-icons/fa';
import { Toolbar } from 'react-science/ui';
import { Multiplets } from '../../../../../../data/constants/Multiplets';
import { isSpectrum1D } from '../../../../../../data/data1d/Spectrum1D';
import { useChartData } from '../../../../../context/ChartContext';
import Button from '../../../../../elements/Button';
import ReactTable from '../../../../../elements/ReactTable/ReactTable';
import FormikInput from '../../../../../elements/formik/FormikInput';
import FormikSelect from '../../../../../elements/formik/FormikSelect';
import { usePanelPreferences } from '../../../../../hooks/usePanelPreferences';
import useSpectrum from '../../../../../hooks/useSpectrum';
import { hasCouplingConstant } from '../../../../../panels/extra/utilities/MultiplicityUtilities';
import { useEvent } from '../../../../../utility/Events';
import { formatNumber } from '../../../../../utility/formatNumber';
const styles = {
    input: {
        width: '100%',
        padding: '0.25rem 0.5rem',
    },
    select: {
        height: '26px',
        width: '100%',
    },
    column: {
        padding: '2px',
    },
};
function getJCouplingKey(signalIndex, jIndex, jCouplingKey) {
    return `signals[${signalIndex}].js.${jIndex}.${jCouplingKey}`;
}
export function SignalJCouplingsTable(props) {
    const { values, setFieldValue } = useFormikContext();
    const signal = values?.signals?.[values?.signalIndex] || {};
    const jCouplings = signal?.js || [];
    const [lastSelectedCouplingIndex, setLastSelectedCouplingIndex] = useState(null);
    const spectrum = useSpectrum();
    const { view: { spectra: { activeTab }, }, } = useChartData();
    const rangesPreferences = usePanelPreferences('ranges', activeTab);
    useEvent({
        onClick: (options) => {
            if (`${props.index}` === values.signalIndex &&
                typeof lastSelectedCouplingIndex === 'number') {
                const x = formatNumber(options.xPPM, rangesPreferences.deltaHz.format);
                void setFieldValue(getJCouplingKey(values.signalIndex, lastSelectedCouplingIndex, 'coupling'), x);
            }
        },
        onBrushEnd: (options) => {
            const { range: [from, to], } = options;
            if (`${props.index}` === values.signalIndex &&
                typeof lastSelectedCouplingIndex === 'number' &&
                isSpectrum1D(spectrum)) {
                const value = Number(formatNumber(Math.abs(to - from) * spectrum.info.originFrequency, rangesPreferences.deltaHz.format));
                void setFieldValue(getJCouplingKey(values.signalIndex, lastSelectedCouplingIndex, 'coupling'), value);
            }
        },
    });
    const addHandler = useCallback((data) => {
        const coupling = {
            multiplicity: translateMultiplet('m'),
            coupling: '',
        };
        void setFieldValue(`signals[${values?.signalIndex}].js`, [
            ...data,
            coupling,
        ]);
        setLastSelectedCouplingIndex(data.length);
    }, [setFieldValue, values?.signalIndex]);
    const deleteHandler = useCallback((data, index) => {
        const jCouplings = data.filter((_, columnIndex) => columnIndex !== index);
        void setFieldValue(`signals[${values?.signalIndex}].js`, jCouplings);
        if (lastSelectedCouplingIndex === index) {
            setLastSelectedCouplingIndex(null);
        }
    }, [lastSelectedCouplingIndex, setFieldValue, values?.signalIndex]);
    function deleteAllHandler() {
        void setFieldValue(`signals[${values?.signalIndex}].js`, []);
        setLastSelectedCouplingIndex(null);
    }
    const multiplicityChangeHandler = useCallback((value, name) => {
        if (!hasCouplingConstant(value)) {
            void setFieldValue(name, '');
        }
    }, [setFieldValue]);
    const COLUMNS = useMemo(() => [
        {
            Header: '#',
            style: { width: '25px', ...styles.column },
            accessor: (_, index) => index + 1,
        },
        {
            Header: 'Multiplicity',
            style: { padding: 0, ...styles.column },
            Cell: ({ row }) => {
                return (_jsx(FormikSelect, { className: "select-box", name: getJCouplingKey(values.signalIndex, row.index, 'multiplicity'), items: Multiplets, itemValueField: "label", onChange: (value) => {
                        multiplicityChangeHandler(value, getJCouplingKey(values.signalIndex, row.index, 'coupling'));
                    }, style: styles.select, onClick: (e) => e.stopPropagation() }));
            },
        },
        {
            Header: 'J (Hz)',
            style: { padding: 0, ...styles.column },
            Cell: ({ row }) => {
                return (_jsx(FormikInput, { name: getJCouplingKey(values.signalIndex, row.index, 'coupling'), type: "number", placeholder: 'J (Hz)', style: { input: styles.input }, disabled: !hasCouplingConstant(row.original?.multiplicity), onClick: (e) => e.stopPropagation() }));
            },
        },
        {
            Header: '',
            style: { width: '70px', ...styles.column },
            id: 'action-button',
            Cell: ({ data, row }) => {
                const record = row.original;
                return (_jsx("div", { style: { display: 'flex', justifyContent: 'space-around' }, children: !record?.name && (_jsx(Button.Danger, { fill: "outline", onClick: (e) => {
                            e.stopPropagation();
                            deleteHandler(data, row.index);
                        }, children: _jsx(FaRegTrashAlt, {}) })) }));
            },
        },
    ], [deleteHandler, multiplicityChangeHandler, values.signalIndex]);
    function selectRowHandler(index) {
        setLastSelectedCouplingIndex((prevIndex) => prevIndex === index ? null : index);
    }
    function handleActiveRow(row) {
        return row?.index === lastSelectedCouplingIndex;
    }
    return (_jsxs("div", { style: { display: 'flex', flexDirection: 'column' }, children: [_jsx("div", { style: { padding: '5px 0', display: 'flex' }, children: _jsxs(Toolbar, { children: [_jsx(Toolbar.Item, { icon: _jsx(FaPlus, {}), tooltip: "Add a new J coupling", intent: "success", onClick: () => addHandler(jCouplings) }), _jsx(Toolbar.Item, { icon: _jsx(FaRegTrashAlt, {}), tooltip: "Delete all J couplings", intent: "danger", onClick: deleteAllHandler })] }) }), _jsx("div", { style: {
                    maxHeight: '300px',
                    overflow: 'auto',
                }, children: _jsx(ReactTable, { data: jCouplings, columns: COLUMNS, onClick: (e, rowData) => selectRowHandler(rowData.index), activeRow: handleActiveRow, emptyDataRowText: "No J Couplings" }) })] }));
}
